"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const apollo_server_core_1 = require("apollo-server-core");
const apollo_server_env_1 = require("apollo-server-env");
const getStreamBody = (stream) => new Promise(resolve => {
    let body;
    stream.on('data', (chunk) => {
        if (!body)
            body = chunk;
        else
            body = Buffer.concat([body, chunk]);
    });
    stream.on('end', () => {
        resolve(body.toString());
    });
});
const createQueryRequest = (headers) => {
    const method = headers[':method'];
    if (!method)
        throw new Error('Missing method');
    const path = headers[':path'];
    if (!path)
        throw new Error('Missing path');
    const rHeaders = new apollo_server_env_1.Headers();
    for (const [name, value] of Object.entries(headers)) {
        if (!value)
            continue;
        if (name.startsWith(':'))
            continue;
        if (Array.isArray(value)) {
            value.forEach(v => rHeaders.append(name, v));
        }
        else {
            rHeaders.append(name, value);
        }
    }
    return new apollo_server_env_1.Request(path, {
        headers: rHeaders,
        method
    });
};
function graphqlHttp2(options) {
    if (!options) {
        throw new Error('Apollo Server requires options.');
    }
    if (arguments.length > 1) {
        throw new Error(`Apollo Server expects exactly one argument, got ${arguments.length}`);
    }
    return async (stream, headers) => {
        const method = headers[':method'];
        if (!method)
            throw new Error('Missing method');
        const path = headers[':path'];
        if (!path)
            throw new Error('Missing path');
        apollo_server_core_1.runHttpQuery([stream, headers], {
            method,
            options: options,
            query: method === 'POST'
                ? JSON.parse(await getStreamBody(stream))
                : new url_1.URL(path).searchParams,
            request: createQueryRequest(headers)
        }).then(({ graphqlResponse, responseInit }) => {
            if (responseInit.headers) {
                stream.respond(responseInit.headers);
            }
            stream.end(graphqlResponse);
        }, (error) => {
            if ('HttpQueryError' !== error.name) {
                throw error;
            }
            stream.respond(Object.assign({}, error.headers, { ':status': error.statusCode }));
            stream.end(error.message);
        });
    };
}
exports.graphqlHttp2 = graphqlHttp2;
//# sourceMappingURL=http2Apollo.js.map